"Use strict";
import "../../node_modules/bootstrap/dist/css/bootstrap.css";
import "../../node_modules/bootstrap-multiselect/dist/css/bootstrap-multiselect.css";
import powerbi from "powerbi-visuals-api";
import {ISelectorManager,ISelectorManagerConstructor, IFilterManager} from "../visual/visualInterfaces";
import * as d3 from "d3";
type Selection<T extends d3.BaseType> = d3.Selection<T, any, any, any>;
/*
负责页面上选择器的展示
*/
export class SelectorManager implements ISelectorManager{
    public selectorContainer : Selection<HTMLElement>;
    private selector:ISelector;
    public filterManager:IFilterManager;
    private categories: powerbi.DataViewCategoryColumn;
    private defaultSelect: powerbi.DataViewValueColumn;
    private defaultStart: powerbi.DataViewValueColumn;
    private defaultEnd: powerbi.DataViewValueColumn;
    constructor(selectorContainer:Selection<HTMLElement>,filterManager:IFilterManager){
        console.debug("selectorManager constructor start");
        this.selectorContainer=selectorContainer;
        this.filterManager=filterManager;
    }
    public dispose():void{
        this.selector.dispose();
        this.selectorContainer.remove();
    }
    public switchSelector<T extends ISelector>(classSelector:new ()=>T){
        let newSelector=new classSelector();
        this.selector?.dispose();
        this.selector=newSelector;
    }
    updateData(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){


        if(!this.selector){
            let fieldType=field.source.type;
            if(fieldType.text){
                this.selector=new DropDownSelector(this);
            }else if(fieldType.numeric){
                this.selector=new DropDownSelector(this);
            }else if(fieldType.dateTime){
                this.selector=new CalendarSelector(this);
            }else{
                throw "only receive text/num/datetime field";
            }
        }
        

        this.selector.update(field, defaultSelect, defaultStart, defaultEnd);
    };
    public filterStringField(selection:string[]){

    };
}

interface ISelector{
    update(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn);
    dispose():void;
}

abstract class Selector implements ISelector{
    protected field: powerbi.DataViewCategoryColumn;
    protected defaultSelect: powerbi.DataViewValueColumn;
    protected defaultStart: powerbi.DataViewValueColumn;
    protected defaultEnd: powerbi.DataViewValueColumn;
    protected checkFieldChange(field: powerbi.DataViewCategoryColumn):boolean{
        //Field is not null, so first check the old field
        if(!this.field){
            return true;
        }
        if(field.source.queryName==this.field.source.queryName){
            console.debug('checkFieldChange: false');
            return false;
        }else{
            console.debug('checkFieldChange: true');
            return true;
        }
    }
    protected abstract checkDefaultSelectionChange(defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn):boolean;
    protected checkFieldAndDefaultSelectionChange(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn){
        console.debug('checkFieldAndDefaultSelectionChange start');
        let flag:boolean=this.checkFieldChange(field)||this.checkDefaultSelectionChange(defaultSelect,defaultStart,defaultEnd)
        console.debug('checkFieldAndDefaultSelectionChange end, result:',flag);
        return flag;
    }
    constructor(manager: ISelectorManager){
        console.debug("Abstract selector:","constructor start");
        this.manager=manager;
        this.createView();
    }
    protected abstract createView():void;
    public abstract dispose():void;
    protected readonly manager:ISelectorManager;
    public abstract update(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn):void;
}
class DropDownSelector extends Selector{
    protected checkDefaultSelectionChange(defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn): boolean {
        //The defaultSelect is nullable, so first check the new defaultSelect
        console.debug('checkDefaultSelectionChange start');
        let flag:boolean;
        if(!defaultSelect||!defaultSelect.values||!defaultSelect.values[0]){
            console.debug('new defaultSelect is null, set flag=false')
            flag=flag||false;
        }else{
            if(!this.defaultSelect||!this.defaultSelect.values||!defaultSelect.values[0]){
                console.debug('previous defaultSelect is null, set flag=true');
                flag=flag||true;
            }else{
                if(defaultSelect.values[0].toString()!=this.defaultSelect.values[0].toString()){
                    console.debug('new defaultSelect not equal to the previous, set flag=true');
                    flag=flag||true;
                }else{
                    console.debug('new defaultSelect equal to the previous, set flag=false');
                    flag=flag||false;
                }
            }
        }
        console.debug('checkDefaultSelectionChange end, result:',flag);
        return flag;
    }
    private dropDown:Selection<HTMLSelectElement>;
    protected createView() {
        console.debug('dropDownViewManager','createView start');
        this.dropDown=this.manager.selectorContainer.append("select").classed("dropDown-selector",true).classed("selector",true);//.attr("multiple",false);
        let filterManager:IFilterManager=this.manager.filterManager;
        this.dropDown.on("input change",function(){
            console.debug('DropDownSelector:',"input change");
            console.debug('this',this);
            console.debug('filterManager:',filterManager);
            let selectedValues:string[]=[];
            for(let i=0;i<this.selectedOptions.length;i++){
                let option=this.selectedOptions.item(i);
                if(option.selected){
                    console.debug("option selected",option);
                    console.debug('option.text:',option.text);
                    selectedValues.push(option.text);
                }
            }
            if(selectedValues.length==0){
            }else{
                console.debug("selection.length",selectedValues.length);
                filterManager.filterStringField(selectedValues);
            }
            console.debug("this",this.selectedOptions);
        });
        console.debug('dropDownViewManager','createView end');

    }
    public dispose(){
        console.debug('dropDown-selector disposing');
        this.dropDown.remove();
        console.debug('dropDown-selector disposed');

    }
    public update(field: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('dropDownViewManager','update start');
        //Check field, defaultSelect are changed or not
        let needUpdateDefaultSelection:boolean=this.checkFieldAndDefaultSelectionChange(field,defaultSelect,defaultStart,defaultEnd);
        let newDefaultSelect:string;
        if(needUpdateDefaultSelection&&defaultSelect&&defaultSelect.values&&defaultSelect.values[0]){
            newDefaultSelect=defaultSelect.values[0].toString();
            console.debug('newDefaultSelect:',newDefaultSelect);
        }
        let options:d3.Selection<HTMLOptionElement, powerbi.PrimitiveValue, HTMLSelectElement, any>=<d3.Selection<HTMLOptionElement, powerbi.PrimitiveValue, HTMLSelectElement, any>>this.dropDown.selectAll("option").data(field.values,function(d){return d.toString();});//map data
        options.exit().remove();//delete
        
        if(needUpdateDefaultSelection&&newDefaultSelect){
            console.debug('reset defaultSelection start');
            options=options.enter().append("option")
                    .text(function(d){return d.toString();})
                    .attr('label',function(d){return d.toString();})
                    .classed('dropDown-option',true)
                    .merge(options)
                    .property("selected",function(d){
                        console.debug('d.toString():',d.toString());
                        console.debug('newDefaultSelect:',newDefaultSelect);
                        return (d.toString()==newDefaultSelect)?'selected':null;
                    });
            this.dropDown.dispatch('change');
            console.debug('reset defaultSelection end');
        }else{
            console.debug('defaultSelection not reset, start update options');
            options.enter().append("option").text(function(d){return d.toString();}).attr('label',function(d){return d.toString();}).classed('dropDown-option',true);//add
            console.debug('defaultSelection not reset, end update options');
        }
        console.debug("dropDown:",this.dropDown);
        this.field=field;
        this.defaultSelect=defaultSelect;
        console.debug('dropDownViewManager','update end');
    }
}
class ListSelector extends Selector{
    protected checkDefaultSelectionChange(defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn): boolean {
        throw new Error("Method not implemented.");
    }
    public dispose() {
        throw new Error("Method not implemented.");
    }
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('ListSelector','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}
class CalendarSelector extends Selector{
    protected checkDefaultSelectionChange(defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn): boolean {
        throw new Error("Method not implemented.");
    }
    public dispose() {
        throw new Error("Method not implemented.");
    }
    protected createView() {
        throw new Error("Method not implemented.");
    }
    public update(categories: powerbi.DataViewCategoryColumn, defaultSelect: powerbi.DataViewValueColumn, defaultStart: powerbi.DataViewValueColumn, defaultEnd: powerbi.DataViewValueColumn) {
        console.debug('calendarViewManager','updateView');
        let dropDownSelector=this.manager.selectorContainer.append("select").classed("listSelector");
    }
}